/*
 * sudo.c
 * Copyright (C) 2008 wantora
 * MIT License
 */

#include <windows.h>
#include <shlwapi.h>
#include <tchar.h>
#include "sudo_server.h"
#include "sudo_client.h"
#include "utils.h"

#define SERVER_MODE  0
#define CLIENT_MODE  1

int _tmain(int argc, TCHAR *argv[]) {
	int i;
	int mode = SERVER_MODE;
	int server_mode = 0;
	LPTSTR command;
	BOOL isSucceeded=TRUE;
	
	for (i = 1; i < argc; i++) {
		if (_tcscmp(argv[i], TEXT("--client")) == 0) {
			mode = CLIENT_MODE;
		} else if (_tcscmp(argv[i], TEXT("--debug")) == 0) {
			server_mode = SERVER_DEBUG_MODE;
		} else {
			break;
		}
	}
	
	if (i >= argc) exit(1);
	
	switch (mode) {
	case SERVER_MODE:
		command = shiftArgs(GetCommandLine(), i);
		isSucceeded = 
                    serverMain((LPCTSTR)argv[0], command, server_mode);
		break;
	case CLIENT_MODE:
		command = shiftArgs(GetCommandLine(), i+1);
		isSucceeded = clientMain((LPCTSTR)argv[i], command);
		break;
	}
	if (!isSucceeded) {
		DWORD lastError=GetLastError();
		TCHAR message[ 1024 ];

		if ( FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM |
							FORMAT_MESSAGE_FROM_SYSTEM ,
							0 ,
							lastError,
							0 ,
							message ,
							sizeof(message)/sizeof(TCHAR) ,
							NULL ) ){
			MessageBox( NULL ,
					    message,
						_T("Error"), 
						MB_OK | MB_ICONINFORMATION);
		}
	}
	
	return 0;
}

#if !defined(_tmain) && defined(UNICODE)
int main() {
	int     argc = 0;
	LPWSTR* argv = CommandLineToArgvW(GetCommandLineW(), &argc);
	return _tmain(argc, argv);
}
#endif
