/*
 * sudo_client.c
 * Copyright (C) 2008 wantora
 * MIT License
 */

#include <windows.h>
#include <shlwapi.h>
#include <wincon.h>
#include <tchar.h>
#include "sudo_client.h"
#include "utils.h"

/* OtpCv̑M */
static void pipeWriter(HANDLE hRead, HANDLE hFile, HANDLE hCEvent, HANDLE hProcess) {
	char szOut[1024];
	DWORD dwRead;
	DWORD dwWritten;
	DWORD dwAvail;
	
	while (1) {
		if (WaitForSingleObject(hCEvent, 10) == WAIT_OBJECT_0) {
			GenerateConsoleCtrlEvent(CTRL_C_EVENT, 0);
			ResetEvent(hCEvent);
		}
		
		if (!PeekNamedPipe(hRead, NULL, 0, NULL, &dwAvail, NULL)) return;
		if (dwAvail > 0) {
			if (!ReadFile(hRead, szOut, sizeof(szOut) - 1, &dwRead, NULL)) return;
			if (dwRead > 0) WriteFile(hFile, szOut, dwRead, &dwWritten, NULL);
		} else if (WaitForSingleObject(hProcess, 0) == WAIT_OBJECT_0) {
			return;
		}
	}
}

/* CTRL_C_EVENT𖳌 */
static BOOL WINAPI HandlerRoutine(DWORD dwCtrlType) {
	if (dwCtrlType == CTRL_C_EVENT) {
		return TRUE;
	}
	
	return FALSE;
}

static void setPath(LPCTSTR env) {
	for (; *env; env++) {
		if (_tcsnicmp(env, TEXT("PATH="), 5) == 0) {
			env += 5;
			SetEnvironmentVariable(TEXT("PATH"), env);
		}
		while (*env) {
			env++;
		}
	}
}

static BOOL clientStart(LPCTSTR pipeName, HANDLE hRead, HANDLE hProcess) {
	HANDLE hFile;
	HANDLE hCEvent;
	
	/*OtpCv*/
	WaitNamedPipe(getPipeName(pipeName), NMPWAIT_WAIT_FOREVER);
	hFile = CreateFile(getPipeName(pipeName), GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile == INVALID_HANDLE_VALUE) return FALSE;
	
	/*ҋ@*/
	SetConsoleCtrlHandler(HandlerRoutine, TRUE);
	hCEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, getCEventName(pipeName));
	
	pipeWriter(hRead, hFile, hCEvent, hProcess);
	WaitForSingleObject(hProcess, INFINITE);
	
	CloseHandle(hCEvent);
	CloseHandle(hFile);
	
	return TRUE;
}

static LPTSTR readEnv(LPCTSTR pipeName) {
	HANDLE hEnvFile;
	DWORD dwRead;
	DWORD dwAvail = 0;
	LPTSTR env;
	
	WaitNamedPipe(getEnvPipeName(pipeName), NMPWAIT_WAIT_FOREVER);
	hEnvFile = CreateFile(getEnvPipeName(pipeName), GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hEnvFile == INVALID_HANDLE_VALUE) return NULL;
	
	while (dwAvail == 0) {
		PeekNamedPipe(hEnvFile, NULL, 0, NULL, &dwAvail, NULL);
		Sleep(0);
	}
	
	env = (LPTSTR)malloc(dwAvail);
	if (env == NULL) return NULL;
	
	ReadFile(hEnvFile, env, dwAvail, &dwRead, NULL);
	CloseHandle(hEnvFile);
	
	return env;
}

/* M */
BOOL clientMain(LPCTSTR pipeName, LPTSTR command) {
	HANDLE hRead, hWrite;
	LPTSTR env;
	DWORD dwCreationFlags;
	
	SECURITY_ATTRIBUTES sa = {0};
	STARTUPINFO si = {0};
	PROCESS_INFORMATION pi = {0};
        BOOL isSucceeded;
	
	/*ϐ̎擾*/
	env = readEnv(pipeName);
	if (env == NULL) return FALSE;
	setPath(env);
	
	/*pCv쐬*/
	sa.nLength = sizeof(sa);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = TRUE;
	if (!CreatePipe(&hRead, &hWrite, &sa, 0)) return FALSE;
	
	SetHandleInformation(hRead, HANDLE_FLAG_INHERIT, 0);
	
	/*CreateProcess*/
	si.cb = sizeof(si);
	si.dwFlags = STARTF_USESTDHANDLES;
	si.hStdOutput = hWrite;
	si.hStdError = hWrite;
	
	dwCreationFlags = 0;
#ifdef UNICODE
	dwCreationFlags |= CREATE_UNICODE_ENVIRONMENT;
#endif
	
	isSucceeded = CreateProcess(NULL, command, NULL, NULL, TRUE, dwCreationFlags, env, NULL, &si, &pi);
	if ( isSucceeded ) {
		CloseHandle(pi.hThread);
		isSucceeded = clientStart(pipeName, hRead, pi.hProcess);
		CloseHandle(pi.hProcess);
	}
	
	free(env);
	CloseHandle(hRead);
	CloseHandle(hWrite);
	return isSucceeded;
}
